#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "iso32i.h"

#define DEVICE "/dev/mem"
#define A460_PC104_IO_8_BASE (0xb2000000)
#define A460_PC104_IO_8_SIZE (0x10000)

static uint8_t *iso32i_base;
#define iso32i_readb(addr) ({*(volatile uint8_t *)(iso32i_base + addr);})

static void print_bin(uintptr_t addr)
{
	uint8_t val;
	int i;

	val = iso32i_readb(addr);
	for (i = ISO32I_CHANNELES_PER_PORT - 1; 0 <= i; i--)
		putchar(!!(val & (1<<i)) + '0');
}

static void print_data(void)
{
	puts("D37..D30 D27..D20 D17..D10 D07..D00");

	print_bin(ISO32I_DAT3);
	putchar(' ');
	print_bin(ISO32I_DAT2);
	putchar(' ');
	print_bin(ISO32I_DAT1);
	putchar(' ');
	print_bin(ISO32I_DAT0);
	putchar('\n');
}

static int xmmap(void)
{
	int fd;

	fd = open(DEVICE, O_RDONLY | O_SYNC);
	if (fd < 0) {
		perror("open");
		return -1;
	}

	iso32i_base = mmap(0x0, A460_PC104_IO_8_SIZE, PROT_READ,
			   MAP_SHARED, fd, A460_PC104_IO_8_BASE);
	if (iso32i_base == MAP_FAILED) {
		perror("mmap");
		close(fd);
		return -1;
	}

	close(fd);

	return 0;
}

static void xmunmap(void)
{
	munmap(iso32i_base, A460_PC104_IO_8_SIZE);
}

int main(void)
{
	int ret;

	ret = xmmap();
	if (ret)
		return EXIT_FAILURE;

	print_data();

	xmunmap();

	return EXIT_SUCCESS;
}
